package com.abi.task.common.tablestore;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.tablestore.common.TableStoreReq;
import com.abi.task.common.tablestore.common.TableStoreRes;
import com.alicloud.openservices.tablestore.SyncClient;
import com.alicloud.openservices.tablestore.model.*;
import com.alicloud.openservices.tablestore.model.search.SearchQuery;
import com.alicloud.openservices.tablestore.model.search.SearchRequest;
import com.alicloud.openservices.tablestore.model.search.SearchResponse;
import com.alicloud.openservices.tablestore.model.search.query.RangeQuery;
import com.alicloud.openservices.tablestore.model.search.query.TermsQuery;
import com.alicloud.openservices.tablestore.model.search.sort.FieldSort;
import com.alicloud.openservices.tablestore.model.search.sort.Sort;
import com.alicloud.openservices.tablestore.model.search.sort.SortOrder;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: fangxinjian
 * @date: 2021/04/15 10:17
 * @description: 表格存储工具类
 */
@Slf4j
@Component
public class TableStoreUtils {

    @Value("${tableStore.endPoint}")
    private String endPoint;

    @Value("${tableStore.accessKeyId}")
    private String accessKeyId;

    @Value("${tableStore.accessKeySecret}")
    private String accessKeySecret;

    @Value("${tableStore.instanceName}")
    private String instanceName;

    /**
     * 创建SyncClient
     *
     * @return
     */
    public SyncClient client() {
        return new SyncClient(endPoint, accessKeyId, accessKeySecret, instanceName);
    }

    /**
     * 创建数据表
     *
     * @param tableName      表名
     * @param primaryKeyName 主键名称
     */
    public void createTable(String tableName, String primaryKeyName) {

        TableMeta tableMeta = new TableMeta(tableName);
        //为主表添加主键列
        tableMeta.addPrimaryKeyColumn(new PrimaryKeySchema(primaryKeyName, PrimaryKeyType.STRING));
        //数据的过期时间，单位为秒，-1表示永不过期。带索引表的数据表数据生命周期必须设置为-1。
        int timeToLive = -1;
        //保存的最大版本数，1表示每列上最多保存一个版本即保存最新的版本。带索引表的数据表最大版本数必须设置为1。
        int maxVersions = 1;
        TableOptions tableOptions = new TableOptions(timeToLive, maxVersions);
        CreateTableRequest request = new CreateTableRequest(tableMeta, tableOptions);
        //设置预留读写吞吐量，容量型实例中的数据表只能设置为0，高性能实例中的数据表可以设置为非零值。
        request.setReservedThroughput(new ReservedThroughput(new CapacityUnit(0, 0)));
        SyncClient client = client();
        client.createTable(request);

    }

    /**
     * 查询所有表的表名
     */
    public List<String> listTable() {
        ListTableResponse listTableResponse = client().listTable();
        return listTableResponse.getTableNames();
    }

    /**
     * 单行插入一行数据
     *
     * @param primaryKeyName 主键名称
     * @param pkValue        主键值
     * @param tableName      表名
     */
    public void putRow(String primaryKeyName, String pkValue, String tableName, List<Column> columns) {
        RowPutChange rowPutChange = buildRowPutChange(primaryKeyName, pkValue, tableName, columns);
        client().putRow(new PutRowRequest(rowPutChange));
    }

    /**
     * 批量插入
     *
     * @param primaryKeyName
     * @param pkValueList
     * @param tableName
     * @param columnsList
     */
    public void batchPutRow(String primaryKeyName, List<String> pkValueList, String tableName, List<List<Column>> columnsList) {
        if (CollectionUtil.isEmpty(pkValueList)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "主键内容为空");
        }
        if (CollectionUtil.isEmpty(columnsList)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "字段列表为空");
        }
        if (pkValueList.size() != columnsList.size()) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "主键与字段数量不匹配");
        }

        BatchWriteRowRequest batchWriteRowRequest = new BatchWriteRowRequest();
        //构造rowPutChange
        for (int i = 0; i < pkValueList.size(); i++) {
            RowPutChange rowPutChange = buildRowPutChange(primaryKeyName, pkValueList.get(i), tableName, columnsList.get(i));
            //添加到batch操作中。
            batchWriteRowRequest.addRowChange(rowPutChange);
        }

        BatchWriteRowResponse response = client().batchWriteRow(batchWriteRowRequest);

        log.info("是否全部成功：{}", response.isAllSucceed());
        if (!response.isAllSucceed()) {
            for (BatchWriteRowResponse.RowResult rowResult : response.getFailedRows()) {
                log.info("失败的行：{}", batchWriteRowRequest.getRowChange(rowResult.getTableName(), rowResult.getIndex()).getPrimaryKey());
                log.info("失败原因：{}", rowResult.getError());
            }
            /**
             * 可以通过createRequestForRetry方法再构造一个请求对失败的行进行重试。此处只给出构造重试请求的部分。
             * 推荐的重试方法是使用SDK的自定义重试策略功能，支持对batch操作的部分行错误进行重试。设置重试策略后，调用接口处无需增加重试代码。
             */
            batchWriteRowRequest.createRequestForRetry(response.getFailedRows());
        }
    }

    /**
     * 单行读取一行数据
     *
     * @param primaryKeyName 主键名称
     * @param pkValue        主键值
     * @param tableName      表名
     * @return 一行数据
     */
    public Row getRow(String primaryKeyName, String pkValue, String tableName) {

        SingleRowQueryCriteria criteria = queryCriteria(primaryKeyName, pkValue, tableName);

        GetRowResponse getRowResponse = client().getRow(new GetRowRequest(criteria));
        return getRowResponse.getRow();
    }

    /**
     * 单行读取一行某一列的数据
     *
     * @param primaryKeyName 主键名称
     * @param pkValue        主键值
     * @param tableName      表名
     * @param columnName     列名称
     * @return 一行某一列的数据
     */
    public Row getColumnRow(String primaryKeyName, String pkValue, String tableName, String columnName) {
        SingleRowQueryCriteria criteria = queryCriteria(primaryKeyName, pkValue, tableName);

        //设置读取某些列。
        criteria.addColumnsToGet(columnName);
        GetRowResponse getRowResponse = client().getRow(new GetRowRequest(criteria));
        return getRowResponse.getRow();

    }

    /**
     * 更新一行数据
     *
     * @param primaryKeyName 主键名称
     * @param pkValue        主键值
     * @param tableName      表名称
     * @param columnName     列名称
     * @param columnValue    列值
     */
    public void updateRow(String primaryKeyName, String pkValue, String tableName, String columnName, String columnValue) {

        RowUpdateChange rowUpdateChange = updateChange(primaryKeyName, pkValue, tableName);
        //更新列数据
        rowUpdateChange.put(new Column(columnName, ColumnValue.fromString(columnValue)));

        client().updateRow(new UpdateRowRequest(rowUpdateChange));
    }

    /**
     * 删除某列数据
     *
     * @param primaryKeyName 主键名称
     * @param pkValue        主键值
     * @param tableName      表名称
     * @param columnName     列名称
     */
    public void deleteColumn(String primaryKeyName, String pkValue, String tableName, String columnName) {

        RowUpdateChange rowUpdateChange = updateChange(primaryKeyName, pkValue, tableName);
        //删除某一列。
        rowUpdateChange.deleteColumns(columnName);

        client().updateRow(new UpdateRowRequest(rowUpdateChange));
    }

    /**
     * 删除一行数据
     *
     * @param primaryKeyName 主键名称
     * @param pkValue        主键值
     * @param tableName      表名称
     */
    public void deleteRow(String primaryKeyName, String pkValue, String tableName) {
        //构造主键。
        PrimaryKeyBuilder primaryKeyBuilder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        primaryKeyBuilder.addPrimaryKeyColumn(primaryKeyName, PrimaryKeyValue.fromString(pkValue));
        PrimaryKey primaryKey = primaryKeyBuilder.build();
        //设置数据表名称。
        RowDeleteChange rowDeleteChange = new RowDeleteChange(tableName, primaryKey);

        client().deleteRow(new DeleteRowRequest(rowDeleteChange));
    }

    /**
     * 通过聚集索引分页查询
     * @param req
     * @return
     */
    public SearchResponse listRows(TableStoreReq req){
        //0-校验入参合法性
        if(StringUtils.isBlank(req.getTableName())){
            throw new BusinessException("TableName为空");
        }
        if(StringUtils.isBlank(req.getIndexName())){
            throw new BusinessException("IndexName为空");
        }
        if(StringUtils.isBlank(req.getFieldName())){
            throw new BusinessException("FieldName为空");
        }
        if(req.getFieldValue()==null
            &&(req.getLessThan()==null || req.getGreaterThan()==null)){
            throw new BusinessException("查询条件为空");
        }

        //1-构建查询条件
        SearchQuery searchQuery = new SearchQuery();

        //精确查询
        if(req.getFieldValue()!=null){
            TermsQuery termsQuery = new TermsQuery();
            //设置要匹配的字段。
            termsQuery.setFieldName(req.getFieldName());
            //设置要匹配的值。
            termsQuery.addTerm(toColumnValue(req.getFieldValue()));
            searchQuery.setQuery(termsQuery);
        }

        //范围匹配
        if(req.getLessThan()!=null && req.getGreaterThan()!=null){
            RangeQuery rangeQuery = new RangeQuery();
            //设置要匹配的字段。
            rangeQuery.setFieldName(req.getFieldName());
            //设置该字段的范围条件为大于小于
            rangeQuery.greaterThan(toColumnValue(req.getGreaterThan()));
            rangeQuery.lessThan(toColumnValue(req.getLessThan()));
            searchQuery.setQuery(rangeQuery);
        }

        //设置按照  列 逆序排序。
        if(req.getSortByFieldDesc()){
            FieldSort fieldSort = new FieldSort(req.getFieldName());
            fieldSort.setOrder(SortOrder.DESC);
            searchQuery.setSort(new Sort(Arrays.asList((Sort.Sorter)fieldSort)));
        }

        //设置分页
        int offset = (req.getPageNo()-1)*req.getPageCount();
        searchQuery.setLimit(req.getPageCount());
        searchQuery.setOffset(offset);

        //设置返回匹配的总行数
        searchQuery.setGetTotalCount(req.getGetTotalCount());

        SearchRequest searchRequest = new SearchRequest(req.getTableName(), req.getIndexName(), searchQuery);
        //通过设置columnsToGet参数可以指定返回的列或返回所有列，如果不设置此参数，则默认只返回主键列。
        SearchRequest.ColumnsToGet columnsToGet = new SearchRequest.ColumnsToGet();
        //设置为返回所有列。
        columnsToGet.setReturnAll(true);
        searchRequest.setColumnsToGet(columnsToGet);
        //table store的出参
        SearchResponse resp = client().search(searchRequest);

        return resp;
    }





    //私有方法-----------------------------------------

    private ColumnValue toColumnValue(Object fieldValue){
        ColumnValue columnValue = null;
        if (fieldValue instanceof String) {
            columnValue = ColumnValue.fromString(String.valueOf(fieldValue));
        } else if (fieldValue instanceof Integer) {
            columnValue = ColumnValue.fromLong(Long.valueOf((Integer) fieldValue));
        } else if (fieldValue instanceof Long) {
            columnValue = ColumnValue.fromLong((Long) fieldValue);
        } else if (fieldValue instanceof Double) {
            columnValue = ColumnValue.fromDouble((Double) fieldValue);
        } else if (fieldValue instanceof Boolean) {
            columnValue = ColumnValue.fromBoolean((Boolean) fieldValue);
        } else if (fieldValue instanceof Date) {
            columnValue = ColumnValue.fromString(DateUtil.format((Date) fieldValue, "yyyy-MM-dd HH:mm:ss"));
        } else if (fieldValue instanceof LocalDateTime) {
            columnValue = ColumnValue.fromString(DateUtil.format((LocalDateTime) fieldValue, "yyyy-MM-dd HH:mm:ss"));
        } else {
            throw new BusinessException("不识别的字段类型fieldValue");
        }

        return columnValue;
    }


    private SingleRowQueryCriteria queryCriteria(String primaryKeyName, String pkValue, String tableName) {
        //构造主键。
        PrimaryKeyBuilder primaryKeyBuilder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        primaryKeyBuilder.addPrimaryKeyColumn(primaryKeyName, PrimaryKeyValue.fromString(pkValue));
        PrimaryKey primaryKey = primaryKeyBuilder.build();

        //读取一行数据，设置数据表名称。
        SingleRowQueryCriteria criteria = new SingleRowQueryCriteria(tableName, primaryKey);
        //设置读取最新版本。
        criteria.setMaxVersions(1);
        return criteria;
    }

    private RowUpdateChange updateChange(String primaryKeyName, String pkValue, String tableName) {
        //构造主键。
        PrimaryKeyBuilder primaryKeyBuilder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        primaryKeyBuilder.addPrimaryKeyColumn(primaryKeyName, PrimaryKeyValue.fromString(pkValue));
        PrimaryKey primaryKey = primaryKeyBuilder.build();
        //设置数据表名称。
        return new RowUpdateChange(tableName, primaryKey);
    }

    /**
     * 构造RowPutChange
     *
     * @param primaryKeyName
     * @param pkValue
     * @param tableName
     * @param columns
     * @return
     */
    private RowPutChange buildRowPutChange(String primaryKeyName, String pkValue, String tableName, List<Column> columns) {
        //构造主键。
        PrimaryKeyBuilder primaryKeyBuilder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        //设置主键名称和主键值
        primaryKeyBuilder.addPrimaryKeyColumn(primaryKeyName, PrimaryKeyValue.fromString(pkValue));
        PrimaryKey primaryKey = primaryKeyBuilder.build();
        //设置数据表名称。
        RowPutChange rowPutChange = new RowPutChange(tableName, primaryKey);

        //添加属性列
        rowPutChange.addColumns(columns);

        return rowPutChange;
    }

    public UpdateTableResponse updateTable(int reservedThroughput, String tableName) {
        UpdateTableRequest updateTableRequest = new UpdateTableRequest();
        updateTableRequest.setTableName(tableName);
        updateTableRequest.setStreamSpecification(new StreamSpecification(true,1));
        ReservedThroughput throughput = new ReservedThroughput();
        throughput.setCapacityUnit(new CapacityUnit(reservedThroughput,reservedThroughput));
        updateTableRequest.setReservedThroughputForUpdate(throughput);
        int timeToLive = -1;
        //保存的最大版本数，1表示每列上最多保存一个版本即保存最新的版本。带索引表的数据表最大版本数必须设置为1。
        int maxVersions = 1;
        TableOptions tableOptions = new TableOptions(timeToLive, maxVersions);
        updateTableRequest.setTableOptionsForUpdate(tableOptions);
        return client().updateTable(updateTableRequest);
    }


    public Map<String,String> listTables() {
        List<String> tableNames = client().listTable().getTableNames();
        if (CollectionUtil.isNotEmpty(tableNames)){
            return tableNames.stream().collect(Collectors.toMap(t->t,t->t));
        }
        return Maps.newHashMap();
    }
}
