package com.abi.task.common.tablestore.common;

import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.IStringUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.checkerframework.checker.units.qual.K;

import java.lang.reflect.Field;

/**
 * 查询table store的入参
 * @author AndyTan
 */
@Data
@Slf4j
public class TableStoreReq<T extends TableStoreEntity> {

    /**
     * 表名
     */
    private String tableName;

    /**
     * 索引名
     */
    private String indexName;

    /**
     * 匹配的字段名
     */
    private String fieldName;

    /**
     * 匹配的字段值
     */
    private Object fieldValue;

    /**
     * 大于xxx
     */
    private Object greaterThan;

    /**
     * 小于xxx
     */
    private Object lessThan;

    /**
     * 每页条数
     */
    private Integer pageCount = 10;

    /**
     * 第几页
     */
    private Integer pageNo = 1;

    /**
     * 需要返回总数量
     */
    private Boolean getTotalCount = false;

    /**
     * 是否逆序
     */
    private Boolean sortByFieldDesc = false;

    /**
     * 印射对象类
     */
    private Class clz;

    /**
     * 静态创建函数
     */
    public static <T extends TableStoreEntity> TableStoreReq<T> build(Class<T> clzParam, String fieldName){
        TableStoreReq<T> req = new TableStoreReq<T>();

        Field field = null;
        try{
            field = clzParam.getDeclaredField(fieldName);
        }catch (Exception e){
            log.info("找不到字段",e);
        }

        //表名
        TableStore tableStore = clzParam.getAnnotation(TableStore.class);
        String tableName = tableStore.tableName();
        //索引
        TableStoreIndex tableStoreIndex = field.getAnnotation(TableStoreIndex.class);
        String indexName = tableStoreIndex.indexName();
        //字段名转下划线
        fieldName = IStringUtil.camelToUnderline(fieldName);

        req.tableName = tableName;
        req.indexName = indexName;
        req.fieldName = fieldName;
        req.clz = clzParam;

        return req;
    }

    /**
     * 构造函数
     * @param tableName 表名
     * @param indexName 索引名
     * @param fieldName 匹配的字段名
     */
    public static <T extends TableStoreEntity> TableStoreReq<T> build(String tableName,String indexName,String fieldName){
        TableStoreReq<T> req = new TableStoreReq<T>();
        req.tableName = tableName;
        req.indexName = indexName;
        req.fieldName = fieldName;
        return req;
    }

    /**
     * 设置范围
     * @param greaterThan
     * @param lessThan
     * @return
     */
    public TableStoreReq<T> range(Object greaterThan,Object lessThan){
        this.setGreaterThan(greaterThan);
        this.setLessThan(lessThan);
        return this;
    }

    /**
     * 设置等于
     * @param fieldValue
     * @return
     */
    public TableStoreReq<T> eq(Object fieldValue){
        this.setFieldValue(fieldValue);
        return this;
    }

    public TableStoreReq<T> page(Integer pageNo,Integer pageCount){
        this.pageNo = pageNo;
        this.pageCount = pageCount;
        return this;
    }

    /**
     * 是否逆序
     * @return
     */
    public TableStoreReq<T> isDesc(){
        this.setSortByFieldDesc(true);
        return this;
    }

    /**
     * 是否需要总数
     * @return
     */
    public TableStoreReq<T> isNeedTotalCount(){
        this.setGetTotalCount(true);
        return this;
    }


}
